/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.example.warehouse.manager.service.dao;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.bluestemsoftware.open.eoa.example.warehouse.manager.vo.CustomerOrder;
import org.bluestemsoftware.open.eoa.example.warehouse.manager.vo.Item;
import org.bluestemsoftware.specification.eoa.application.spring.MyApplication;
import org.bluestemsoftware.specification.eoa.application.spring.MyApplicationContext;

public class CustomerOrderDAO implements DataAccessObject {

    private Map<String, CustomerOrder> customerOrders;
    private String orderNumberPrefix;
    
    public void init() {
        
        // using name of engine which implements this application
        // we derive the order prefix which indicates warehouse
        // from which order is fulfilled.
        
        MyApplication myApplication = MyApplicationContext.getApplication();
        QName engineName = myApplication.getName();
        orderNumberPrefix = engineName.getNamespaceURI().substring(25,28).toUpperCase();
        
    }
    
    private CustomerOrderDAO() {
        
        // TODO: replace map with a datasource injected using
        // spring's jee:jndi-lookup tag once ldap directory
        // server feature is implemented (which will bind all
        // connectors by default)
        
        customerOrders = new HashMap<String, CustomerOrder>();
        
    }
    
    public synchronized CustomerOrder createCustomerOrder(String zipCode, Set<Item> items, double shipCost) {
        
        // TODO: insert a customer order record into db where
        // order number is pk, status is a field and using
        // two tables that model the one to many relationship
        // between customer order and items, insert the
        // items
        
        String orderNumber = createOrderNumber();
        CustomerOrder co = new CustomerOrder(orderNumber, zipCode, items, shipCost);
        customerOrders.put(orderNumber, co);
        return co;
        
    }
    
    public synchronized CustomerOrder getCustomerOrder(String orderNumber) {
        return customerOrders.get(orderNumber);
    }
    
    public synchronized void updateCustomerOrder(CustomerOrder customerOrder) {
        
        // TODO: update customer order record in db, i.e.
        // set status, shipping price, etc ... where order
        // number is ... and insert/update any outstanding
        // requests
        
        customerOrders.put(customerOrder.getOrderNumber(), customerOrder);
        
    }
    
    private String createOrderNumber() {
        
        // the example uses a 'standard' order number format to
        // demonstrate the use of message properties. it consists
        // of three chars followed by a dash and exactly five
        // digits ... 
        
        double random = (Math.random() * Integer.parseInt("99998"));
        String digits = Long.toString(Math.round(random));
        while(digits.length() < 5) {
            digits = "0" + digits;
        }
        
        return orderNumberPrefix + "-" + digits;
        
    }
    
}
